#include "lib/gba.h"
#include "res.h"

#define BG_MAX_CNT 4

typedef struct {
	u32  mapBase;
	u16* mapBaseAdr;
	u32  tileBase;
	u16* tileBaseAdr;
} ST_BG;

//---------------------------------------------------------------------------
ST_BG Bg[BG_MAX_CNT];

//---------------------------------------------------------------------------
void WaitForVsync(void)
{
	while(*(vu16*)0x4000006 >= 160) {};
	while(*(vu16*)0x4000006 <  160) {};
}
//---------------------------------------------------------------------------
void BgInitMem(void)
{
	const u32 mapBase[]  = { 11, 12,  0,  0 };
	const u32 tileBase[] = {  0,  2,  0,  0 };
	s32 i;

	for(i=0; i<BG_MAX_CNT; i++)
	{
		Bg[i].mapBase     = MAP_BASE(mapBase[i]);
		Bg[i].mapBaseAdr  = MAP_BASE_ADR(mapBase[i]);
		Bg[i].tileBase    = TILE_BASE(tileBase[i]);
		Bg[i].tileBaseAdr = TILE_BASE_ADR(tileBase[i]);
	}

	for(i=0; i<32*32; i++)
	{
		Bg[0].mapBaseAdr[i] = 0;
		Bg[1].mapBaseAdr[i] = 0;
		Bg[2].mapBaseAdr[i] = 0;
		Bg[3].mapBaseAdr[i] = 0;
	}

	for(i=0; i<0x2000; i++)
	{
		Bg[0].tileBaseAdr[i] = 0;
		Bg[1].tileBaseAdr[i] = 0;
		Bg[2].tileBaseAdr[i] = 0;
		Bg[3].tileBaseAdr[i] = 0;
	}
}
//---------------------------------------------------------------------------
void BgInit(void)
{
	BgInitMem();

	REG_DISPCNT = (MODE_0 | BG0_ON | BG1_ON | WIN0_ENABLE);

	REG_BG0CNT = (BG_SIZE_0 | BG_16_COLOR | Bg[0].tileBase | Bg[0].mapBase);
	REG_BG1CNT = (BG_SIZE_0 | BG_16_COLOR | Bg[1].tileBase | Bg[1].mapBase);
}
//---------------------------------------------------------------------------
void BgSetTile(u32 bg, u16* pDat, u32 size)
{
	volatile u32 i;

	for(i=0; i<size; i++)
	{
		Bg[bg].tileBaseAdr[i] = pDat[i];
	}
}
//---------------------------------------------------------------------------
void BgSetPal(u32 pal, u16* pDat)
{
	volatile u32 i;

	for(i=0; i<16; i++)
	{
		BG_PALETTE[pal*16+i] |= pDat[i];
	}
}
//---------------------------------------------------------------------------
void BgSetMap(void)
{
	volatile u32 i;

	for(i=0; i<32*20; i++)
	{
		Bg[0].mapBaseAdr[i] = i;
		Bg[1].mapBaseAdr[i] = i | (1 << 12);
	}
}
//---------------------------------------------------------------------------
int main(void)
{
	BgInit();

	BgSetTile(0, (u16*)&imageTiles, imageTilesLen/2);
	BgSetTile(1, (u16*)&image2Tiles, image2TilesLen/2);

	BgSetPal(0, (u16*)&imagePal);
	BgSetPal(1, (u16*)&image2Pal);

	BgSetMap();


	// EBhE̒BG1,OBG0\
	REG_WININ  = WIN_0_BG1;
	REG_WINOUT = WIN_0_BG0;

	s32 ofx = 20;
	s32 ofy = 20;

	for(;;)
	{
		WaitForVsync();

		REG_WIN0H = WIN_LEFT(ofx) | WIN_RIGHT(ofx+64);
		REG_WIN0V = WIN_TOP(ofy)  | WIN_DOWN(ofy+64);

		if( !(REG_KEYINPUT & KEY_UP)    ) ofy--;
		if( !(REG_KEYINPUT & KEY_DOWN)  ) ofy++;
		if( !(REG_KEYINPUT & KEY_LEFT)  ) ofx--;
		if( !(REG_KEYINPUT & KEY_RIGHT) ) ofx++;
	}
}
